﻿using DotNetCore.CAP;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using MapsterMapper;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Sharing.ConvergenceMedia.Dtos;
using Sharing.Orders;
using Sharing.Share.Dtos.ConvergenceMedia;
using Sharing.WebCore;
using SqlSugar;
using XF.Domain.Repository;

namespace Sharing.ConvergenceMedia.Controllers
{
    public class ConvergenceMediaController : BaseController
    {
        private readonly IMapper _mapper;
        private readonly ICapPublisher _capPublisher;
        private readonly IChannelConfigurationManager _channelConfigurationManager;
        private readonly IRepository<MediaDataReceive> _mediaDataReceiveRepository;
        private readonly IDataOrderRepository _dataOrderRepository;
        private readonly IRepository<DataOrderWorkFlow> _dataOrderWorkFlowRepository;

        /// <summary>
        /// 
        /// </summary>
        /// <param name="mapper"></param>
        /// <param name="capPublisher"></param>
        /// <param name="channelConfigurationManager"></param>
        /// <param name="mediaDataReceiveRepository"></param>
        /// <param name="dataOrderRepository"></param>
        /// <param name="dataOrderWorkFlowRepository"></param>
        public ConvergenceMediaController(IMapper mapper, ICapPublisher capPublisher, IChannelConfigurationManager channelConfigurationManager,
            IRepository<MediaDataReceive> mediaDataReceiveRepository, IDataOrderRepository dataOrderRepository, IRepository<DataOrderWorkFlow> dataOrderWorkFlowRepository)
        {
            _mapper = mapper;
            _capPublisher = capPublisher;
            _channelConfigurationManager = channelConfigurationManager;
            _mediaDataReceiveRepository = mediaDataReceiveRepository;
            _dataOrderRepository = dataOrderRepository;
            _dataOrderWorkFlowRepository = dataOrderWorkFlowRepository;
        }

        /// <summary>
        /// 2.1融媒体写信
        /// </summary>
        /// <param name="mediaDataReceiveDto"></param>
        /// <returns></returns>
        [HttpPost("WriteOrder")]
        [AllowAnonymous]
        public async Task<List<DeReponse<WriteOrderResponseDto>>> WriteOrder([FromBody] MediaDataReceiveDto mediaDataReceiveDto)
        {
            //验证密钥
            var mediaCode = _channelConfigurationManager.GetConfigurationMediaCode();
            if (mediaCode.MediaCode.ToLower().Trim() != mediaDataReceiveDto.Code.ToLower().Trim())
                return DeReponse<WriteOrderResponseDto>.Failed(new WriteOrderResponseDto() { AcceptCode = "", AcceptPwd = "", msg = "密钥错误", Code = "-900" }, "1", "密钥错误");

            //验证数据
            string strResult = mediaDataReceiveDto.Validate();
            if (!string.IsNullOrEmpty(strResult))
                return DeReponse<WriteOrderResponseDto>.Failed(new WriteOrderResponseDto() { AcceptCode = "", AcceptPwd = "", msg = strResult, Code = "0" }, "1", description: strResult);

            //转换数据
            var dto = _mapper.Map<MediaDataReceive>(mediaDataReceiveDto);
            //添加数据
            var id = await _mediaDataReceiveRepository.AddAsync(dto, HttpContext.RequestAborted);
            if (!string.IsNullOrEmpty(id))
            {
                OrderDto data = new()
                {
                    Title = mediaDataReceiveDto.Title,
                    FromName = mediaDataReceiveDto.LinkName,
                    Contact = mediaDataReceiveDto.Mobile,
                    Address = mediaDataReceiveDto.Address,
                    Content = mediaDataReceiveDto.Content,
                    IsSecret = mediaDataReceiveDto.Secrecy == 0 ? false : true,
                    Source = Hotline.Share.Enums.Order.ESource.ConvergenceMedia,
                    FromGender = mediaDataReceiveDto.Gender switch
                    {
                        "1" => Hotline.Share.Enums.Order.EGender.Male,
                        "2" => Hotline.Share.Enums.Order.EGender.Female,
                        _ => Hotline.Share.Enums.Order.EGender.Unknown,
                    }
                };
                switch (mediaDataReceiveDto.PurTypeName)
                {
                    case 17:
                        data.AcceptType = "咨询";
                        data.AcceptTypeCode = "10";
                        break;
                    case 18:
                        data.AcceptType = "举报";
                        data.AcceptTypeCode = "30";
                        break;
                    case 19:
                        data.AcceptType = "投诉";
                        data.AcceptTypeCode = "35";
                        break;
                    case 20:
                        data.AcceptType = "求助";
                        data.AcceptTypeCode = "20";
                        break;
                    case 21:
                        data.AcceptType = "建议";
                        data.AcceptTypeCode = "15";
                        break;
                    case 25:
                        data.AcceptType = "其他";
                        data.AcceptTypeCode = "40";
                        break;
                    default:
                        data.AcceptType = "其他";
                        data.AcceptTypeCode = "40";
                        break;
                }

                string areaName = "";
                switch (mediaDataReceiveDto.AreaID)
                {
                    case 2:
                        data.AreaCode = "511504"; areaName = "叙州区";
                        break;
                    case 3:
                        data.AreaCode = "511503"; areaName = "翠屏区";
                        break;
                    case 4:
                        data.AreaCode = "511504"; areaName = "南溪区";
                        break;
                    case 5:
                        data.AreaCode = "511524"; areaName = "长宁县";
                        break;
                    case 6:
                        data.AreaCode = "511528"; areaName = "兴文县";
                        break;
                    case 7:
                        data.AreaCode = "511526"; areaName = "珙县";
                        break;
                    case 8:
                        data.AreaCode = "511525"; areaName = "高县";
                        break;
                    case 9:
                        data.AreaCode = "511527"; areaName = "筠连县";
                        break;
                    case 10:
                        data.AreaCode = "511523"; areaName = "江安县";
                        break;
                    case 11:
                        data.AreaCode = "511529"; areaName = "屏山县";
                        break;
                    case 12:
                        data.AreaCode = "511504"; areaName = "叙州区";
                        break;
                    default:
                        break;
                }
                data.Address = "四川省宜宾市" + areaName;
                data.Password = Convert.ToString(DateTime.Now.Second.ToString() + DateTime.Now.Millisecond.ToString() + "0000").Substring(0, 4);

                //向业务系统推送消息
                await _capPublisher.PublishAsync(Sharing.Share.Mq.EventNames.SharingOrderCreation, data, cancellationToken: HttpContext.RequestAborted);

                return DeReponse<WriteOrderResponseDto>.Success(new WriteOrderResponseDto() { AcceptCode = "", AcceptPwd = data.Password, msg = "写信成功", Code = "1" }, "您已成功提交数据！");
            }
            else
                return DeReponse<WriteOrderResponseDto>.Failed(new WriteOrderResponseDto() { AcceptCode = "", AcceptPwd = "", msg = "接口调用失败", Code = "0" }, "1", description: "接口调用失败！");
        }

        /// <summary>
        /// 2.2工单列表
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("GetOrderByList")]
        [AllowAnonymous]
        public async Task<List<DeReponse<GetOrderMediaDataDto>>> GetOrderByList([FromBody] GetOrderMediaReceiveDto dto)
        {
            //密钥验证
            var mediaCode = _channelConfigurationManager.GetConfigurationMediaCode();
            if (mediaCode.MediaCode.ToLower().Trim() != dto.Code.ToLower().Trim())
                return DeReponse<GetOrderMediaDataDto>.Failed(null, "0", "密钥错误");

            //数据处理
            string AcceptTypeCode = "";
            switch (dto.PurTypeID)
            {
                case 17:
                    AcceptTypeCode = "10";
                    break;
                case 18:
                    AcceptTypeCode = "30";
                    break;
                case 19:
                    AcceptTypeCode = "35";
                    break;
                case 20:
                    AcceptTypeCode = "20";
                    break;
                case 21:
                    AcceptTypeCode = "15";
                    break;
                case 25:
                    AcceptTypeCode = "40";
                    break;
                default:
                    AcceptTypeCode = "";
                    break;
            }

            //数据查询
            RefAsync<int> total = 0;
            var items = await _dataOrderRepository
            .Queryable()
            .Where(p => p.Source == ESource.ConvergenceMedia.ToString())
            .WhereIF(!string.IsNullOrEmpty(dto.AcceptCode), p => p.OrderNo.Contains(dto.AcceptCode))
            .WhereIF(!string.IsNullOrEmpty(dto.Title), p => p.Title.Contains(dto.Title))
            .WhereIF(!string.IsNullOrEmpty(dto.Mobile), p => p.Contact.Contains(dto.Mobile))
            .WhereIF(!string.IsNullOrEmpty(AcceptTypeCode), p => p.CaseTypeCode == AcceptTypeCode)
            .WhereIF(!string.IsNullOrEmpty(dto.StartDate), p => p.CaseDate >= Convert.ToDateTime(dto.StartDate))
            .WhereIF(!string.IsNullOrEmpty(dto.EndDate), p => p.CaseDate < Convert.ToDateTime(dto.EndDate).AddDays(1))
            .Select(it => new
            {
                FlowID = it.Id,
                RowID = SqlFunc.MappingColumn(default(int), "  row_number()  over( order by 'AddDate' desc ) "),
                //  RowID= SqlFunc.RowNumber(it.CaseDate),
                Code = it.OrderNo,
                Title = it.Title,
                AddDate = it.CaseDate,
                FromName = it.CaseSource,
                PurTypeName = it.CaseType,
                ResultState = it.HandleState
            })
              .ToPageListAsync(dto.CurrentPage, dto.PageSize, total);

            //计算总页数
            int nPageCount = Convert.ToInt32(Math.Ceiling(Convert.ToDouble(total) / dto.PageSize));

            //处理返回
            GetOrderMediaDataDto dataDto = new()
            {
                PageCount = nPageCount,
                data = _mapper.Map<IReadOnlyList<GetOrderMediaDataListDto>>(items)
            };

            return DeReponse<GetOrderMediaDataDto>.Success(dataDto, "成功");

        }

        /// <summary>
        /// 2.3 工单查询
        /// </summary>
        /// <returns></returns>
        [HttpPost("GetOrderDetail")]
        [AllowAnonymous]
        public async Task<List<DeReponse<IReadOnlyList<MediaOrderDetailDataDto>>>> GetOrderDetail([FromBody] MediaOrderDetailReceiveDto dto)
        {
            //验证密钥
            var mediaCode = _channelConfigurationManager.GetConfigurationMediaCode();
            if (mediaCode.MediaCode.ToLower().Trim() != dto.Code.ToLower().Trim())
                return DeReponse<IReadOnlyList<MediaOrderDetailDataDto>>.Failed(null, "0", "密钥错误");

            //验证数据
            string strResult = dto.Validate();
            if (!string.IsNullOrEmpty(strResult))
                return DeReponse<IReadOnlyList<MediaOrderDetailDataDto>>.Failed(null, "0", strResult);

            //查询数据
            var items = await _dataOrderRepository
             .Queryable()
             .Where(p => p.OrderNo == dto.AcceptCode)
             .Where(p => p.Password == dto.AcceptPwd)
              .Select(it => new
              {
                  FlowID = it.Id,
                  FlowCode = it.OrderNo,
                  FlowTitle = it.Title,
                  FlowFromName = it.CaseSource,
                  FlowPurTypeName = it.CaseType,
                  FlowConTypeName = it.HotspotName,
                  FlowAddDate = it.CaseDate,
                  FlowLKName = it.FromName,
                  FlowBMName = it.ActualHandleOrgName,
                  FlowRSFlagName = it.HandleState,
                  FlowContent = it.Content,
                  FlowResult = it.ActualOpinion,
              })
              .ToListAsync();

            if (items == null)
                return DeReponse<IReadOnlyList<MediaOrderDetailDataDto>>.Failed(null, "0", "编号、密码错误！");
            else
                return DeReponse<IReadOnlyList<MediaOrderDetailDataDto>>.Success(_mapper.Map<IReadOnlyList<MediaOrderDetailDataDto>>(items), "成功");

        }

        /// <summary>
        /// 2.4 今日受理分类统计
        /// </summary>
        /// <param name="Code"></param>
        /// <returns></returns>
        [HttpPost("GetPurTypeReport")]
        [AllowAnonymous]
        public async Task<List<DeReponse<List<GetPurTypeReportDataDto>>>> GetPurTypeReport([FromBody] string Code)
        {
            //验证密钥
            var mediaCode = _channelConfigurationManager.GetConfigurationMediaCode();
            if (mediaCode.MediaCode.ToLower().Trim() != Code.ToLower().Trim())
                return DeReponse<List<GetPurTypeReportDataDto>>.Failed(null, "0", "密钥错误");

            //数据查询
            var list = _dataOrderRepository.Queryable()
                .Where(p => p.CaseDate >= Convert.ToDateTime(DateTime.Now.ToString("yyyy-MM-dd 00:00:00")))
                .Where(p => p.CaseDate <= Convert.ToDateTime(DateTime.Now.ToString("yyyy-MM-dd 23:59:59")))
                .Select(it => new
                {
                    it.CaseType,
                    it.CaseTypeCode
                })
                .MergeTable()//将查询出来的结果合并成一个新表
                 .GroupBy(it => new { it.CaseType, it.CaseTypeCode })//对新表进行分组
                 .Select(it => new
                 {
                     PurTypeName = it.CaseType,
                     Count = SqlFunc.AggregateCount(it.CaseTypeCode)
                 })
                 .ToListAsync();

            var listData = _mapper.Map<List<GetPurTypeReportDataDto>>(list);
            return DeReponse<List<GetPurTypeReportDataDto>>.Success(listData, "成功");
        }

        /// <summary>
        /// 2.5 答复公开
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("GetOrderByListOpen")]
        [AllowAnonymous]
        public async Task<List<DeReponse<GetOrderMediaDataDto>>> GetOrderByListOpen([FromBody] GetOrderMediaReceiveOpenDto dto)
        {
            //密钥验证
            var mediaCode = _channelConfigurationManager.GetConfigurationMediaCode();
            if (mediaCode.MediaCode.ToLower().Trim() != dto.Code.ToLower().Trim())
                return DeReponse<GetOrderMediaDataDto>.Failed(null, "0", "密钥错误");

            //数据处理
            string AcceptTypeCode = "";
            switch (dto.PurTypeID)
            {
                case 17:
                    AcceptTypeCode = "10";
                    break;
                case 18:
                    AcceptTypeCode = "30";
                    break;
                case 19:
                    AcceptTypeCode = "35";
                    break;
                case 20:
                    AcceptTypeCode = "20";
                    break;
                case 21:
                    AcceptTypeCode = "15";
                    break;
                case 25:
                    AcceptTypeCode = "40";
                    break;
                default:
                    AcceptTypeCode = "";
                    break;
            }

            //查询数据
            RefAsync<int> total = 0;
            var items = await _dataOrderRepository
            .Queryable()
            .Where(p => p.IsPublish == Sharing.Share.Enums.EPublishState.Open)
            .WhereIF(!string.IsNullOrEmpty(dto.Title), p => p.Title.Contains(dto.Title))
            .WhereIF(!string.IsNullOrEmpty(AcceptTypeCode), p => p.CaseTypeCode == AcceptTypeCode)
            .WhereIF(!string.IsNullOrEmpty(dto.StartDate), p => p.CaseDate >= Convert.ToDateTime(dto.StartDate))
            .WhereIF(!string.IsNullOrEmpty(dto.EndDate), p => p.CaseDate < Convert.ToDateTime(dto.EndDate).AddDays(1))
            .Select(it => new
            {
                FlowID = it.Id,
                RowID = SqlFunc.MappingColumn(default(int), "  row_number()  over( order by 'AddDate' desc ) "),
                Code = it.OrderNo,
                Title = it.Title,
                AddDate = it.CaseDate,
                FromName = it.CaseSource,
                PurTypeName = it.CaseType,
                ResultState = it.HandleState
            })
              .ToPageListAsync(dto.CurrentPage, dto.PageSize, total);

            //计算总页数
            int nPageCount = Convert.ToInt32(Math.Ceiling(Convert.ToDouble(total) / dto.PageSize));

            //返回数据处理
            GetOrderMediaDataDto dataDto = new()
            {
                PageCount = nPageCount,
                data = _mapper.Map<IReadOnlyList<GetOrderMediaDataListDto>>(items)
            };

            return DeReponse<GetOrderMediaDataDto>.Success(dataDto, "成功");

        }

        /// <summary>
        /// 2.6 办理明细
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("GetOrderByRun")]
        [AllowAnonymous]
        public async Task<List<DeReponse<IReadOnlyList<GetOrderByRunDataDto>>>> GetOrderByRun([FromBody] MediaOrderDetailReceiveDto dto)
        {
            //密钥验证
            var mediaCode = _channelConfigurationManager.GetConfigurationMediaCode();
            if (mediaCode.MediaCode.ToLower().Trim() != dto.Code.ToLower().Trim())
                return DeReponse<IReadOnlyList<GetOrderByRunDataDto>>.Failed(null, "0", "密钥错误");

            //验证数据
            string strResult = dto.Validate();
            if (!string.IsNullOrEmpty(strResult))
                return DeReponse<IReadOnlyList<GetOrderByRunDataDto>>.Failed(null, "0", strResult);

            //验证信件是否存在
            var data = await _dataOrderRepository.GetAsync(p => p.OrderNo == dto.AcceptCode && p.Password == dto.AcceptPwd);
            if (data == null)
                return DeReponse<IReadOnlyList<GetOrderByRunDataDto>>.Failed(null, "0", "编号、密码错误！");

            //数据查询
            var list = await _dataOrderWorkFlowRepository.Queryable()
                  .Where(p => p.OrderId == data.OrderId)
                  .Where(p => p.WorkflowId == data.WorkflowId)
                  .Select(it => new
                  {
                      BMName = it.HandlerOrgName,
                      TTime = it.StepExpiredTime,
                      DisposeDate = it.HandleTime
                  })
                  .ToListAsync();
            return DeReponse<IReadOnlyList<GetOrderByRunDataDto>>.Success(_mapper.Map<IReadOnlyList<GetOrderByRunDataDto>>(list), "成功");
        }

        /// <summary>
        /// 2.1.3 上传附件----未完成
        /// </summary>
        /// <param name="formData"></param>
        /// <returns></returns>
        [HttpPost("WriteFiles")]
        [AllowAnonymous]
        public async Task<byte[]> WriteFiles([FromForm] IFormCollection formData)
        {
            string Code = formData["Code"];
            string AcceptCode = formData["AcceptCode"];
            string AcceptPwd = formData["AcceptPwd"];

            IFormFileCollection files = formData.Files;//等价于Request.Form.Files
            long size = files.Sum(f => f.Length);
            var file = files[0];

            string FileName = file.FileName;

            var stream = file.OpenReadStream();
            byte[] ByteFile = i_StreamToBytes(stream);
            return ByteFile;
        }


        #region 私有方法-将文件转化为文件流
        /// <summary>
        /// 将文件转化为文件流
        /// </summary>
        /// <param name="stream"></param>
        /// <returns></returns>
        private byte[] i_StreamToBytes(Stream stream)
        {
            byte[] bytes = new byte[stream.Length];
            stream.Read(bytes, 0, bytes.Length);
            // 设置当前流的位置为流的开始    
            stream.Seek(0, SeekOrigin.Begin);
            return bytes;
        }
        #endregion
    }
}
